/*******************************************************************************
 * Copyright (c) 2000, 2022 IBM Corporation and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.gef.examples.logicdesigner.model.commands;

import org.eclipse.gef.examples.logicdesigner.LogicMessages;
import org.eclipse.gef.examples.logicdesigner.model.LogicDiagram;
import org.eclipse.gef.examples.logicdesigner.model.LogicSubpart;

public class AddCommand extends org.eclipse.gef.commands.Command {

	private LogicSubpart child;
	private LogicDiagram parent;
	private int index = -1;

	public AddCommand() {
		super(LogicMessages.AddCommand_Label);
	}

	@Override
	public void execute() {
		if (index < 0) {
			parent.addChild(child);
		} else {
			parent.addChild(child, index);
		}
	}

	public LogicDiagram getParent() {
		return parent;
	}

	@Override
	public void redo() {
		if (index < 0) {
			parent.addChild(child);
		} else {
			parent.addChild(child, index);
		}
	}

	public void setChild(LogicSubpart subpart) {
		child = subpart;
	}

	public void setIndex(int i) {
		index = i;
	}

	public void setParent(LogicDiagram newParent) {
		parent = newParent;
	}

	@Override
	public void undo() {
		parent.removeChild(child);
	}

}
